
enum {
	SSIconStyleNoIcon,         /*!<  Alert will have no icon */
	SSIconStyleCritical,       /*!<  Alert will have "critical" (red) icon */
	SSIconStyleInformational   /*!<  Alert will have "informational" (yellow) icon */
} ;

/*
 @brief    This class provides a "shared" (singleton) alert dialog which is similar to Apple's NSRunXXXXAlert but
 has additional features.

 @detail
 <p>
 The additional features are:
 <ul>
 <li>-  Optional progress bar</li>
 <li>-  Optional checkbox which may be used for stuff like "Don't show this again".</li>
 <li>-  Optional Help button.</li>
 <li>-  Optional Support button (Creates a new email message to designated support email address.)</li>
 <li>-  Optional text field(s).</li>
 <li>-  Optional popup menu(s).</li>
 <li>-  Can be run as modal session as well as modal dialog.</li>
 <li>-  Same alert can be reconfigured on the fly to show progress of cascaded or nested processes, which looks much better than having alerts rapidly appear and disappear.</li>
 <li>-  A push/poppable "stack" of configurations remembers nested configurations.</li>
 <li>-  Relatively foolproof "identifiers" help to debug configuration stack mismatches.</li>
 <li>-  Also a replacement for -[NSError presentError:].  Use +[SSAlert alertError:].</li>
 </ul>
 </p>
 
 
 <p>
 Alerts similar to NSRunXXXAlert and -[NSError presentError:] can be run using one of 
 the "all in one" class methods.  For more complicated alerts, do this....
 </p>
 
 <p>
 <ul>
 <li>1.  SSAlert* alert = [SSAlert sharedAlert]</li>
 <li>1a. [Optional] If desired to push the previous configuration onto the stack,</li>
 <li>	NSString* prevAlertConfig = [alert pushConfiguration:@"My1234Configuration"] ;</li>
 <li>2.	Send configuration messages.</li>
 <li>3.  -display  (Calculates layout and displays)</li>
 <li>4a. To run as modal dialog or modal session, </li>
 <li>    -runModalDialog (can only be ended by clicking a button)</li>
 <li>    -runModalSession</li>
 <li>4b. To run as regular window which will retain itself</li>
 <li>	and go away when a button is clicked, send</li>
 <li>	-setGoAwayWhenButtonClicked:YES</li>
 <li>5.  To end a modal session without causing alert to go away,</li>
 <li>	-endModalSession.</li>
 <li>7a. When done, -goAway, unless you have -setGoAwayWhenClicked:YES.</li>
 <li>7b. If you did 1a,</li>
 <li>    [alert popConfiguration:prevAlertConfig] ;</li>
 </ul>
 </p>
 
 <p>
 In utility methods, when you are not sure whether or not there is a previous configuration to push, you may use 1a and 7a, because if the alert is not visible, 1a will not push anything onto the stack and will return nil, and with argument nil, 7a will not pop anything off the stack but will simply invoke -goAway.
 </p>
 
 <p>
 If you start another modal session after this one is started, say, 
 for example, with an NSAlert, i.e. nested modal sessions, and then stop that
 inner modal session, then the modal session in this progress SSAlert will also
 return YES to -done because -runModalSession will return NSRunStoppedResponse.
 Apparently, any nested modal session will "stop" outer sessions.  Makes sense.
 But, I do not know of any way to "restart" this modal session.  You might try
 using -stopModalWithCode:NSRunContinuesResponse, but that would be a kludge,
 and I was getting crashes when using stopModal while developing this,
 so I said "Forget it".  Bottom line: When using the modal session in here
 (cancellable:YES), you canNOT stack another modal session on top of it and
 expect this one to continue after the one of top of it ends. 
 </p> 
 */
@interface SSAlert : NSObject {
	NSWindow* _window ;
    
	// Subviews, all created programatically
    NSImageView* _icon ;
	NSProgressIndicator* _progressBar ;
    NSTextView* _titleTextView ;
    NSTextView* _smallTextView ;
	NSButton* _helpButton ;
	NSButton* _supportButton ;
	NSButton* _checkbox ;
	NSButton* _button1 ;
    NSButton* _button2 ;
    NSButton* _button3 ;
	NSMutableArray* _otherSubviews ;
	
	NSMutableArray* _configurations ;	

	NSString* _helpAnchorString ;
	NSString* _supportEmailString ;
	NSError* _errorPresenting ;

    int _nDone ;
	int _alertReturn ;
	float _minTextFieldWidth ;
	BOOL _allowsShrinking ;
	int _titleMaxChars ;
	int _smallTextMaxChars ;

	BOOL _goAwayWhenButtonClicked ;
	BOOL _doingModalDialog ;
	NSModalSession _modalSession ;
	NSPoint _windowTopCenter ;
	
	// Objects stolen from GetAlertPanel
	NSImageView* _iconInformational ;
	NSImageView* _iconCritical ;
	NSTextField* _titleTextFieldPrototype ;
	NSTextField* _smallTextFieldPrototype ;
	NSButton* _buttonPrototype ;
	NSString* _wordAlert ;
	
	BOOL _progressBarShouldAnimate ;
	
	BOOL _doNotDisplayNextPop ;
	
	NSTimeInterval _nextProgressUpdate ;
}


// Creating
/*!
 @brief    Designated initializer for the SSAlert class.
 @result   An initialized instance of SSAlert.
*/
- (id)init ;

/*!
 @brief    Convenience method for returning the application-wide sharedAlert
 instance of NSAlert.

 @details  Using this singleton sharedAlert will avoid creating a stack of
 dialogs in the event of nested events generating alerts
 @result   The application-wide sharedAlert, autoreleased
*/
+ (SSAlert*)sharedAlert ;

#pragma mark * All-In-One Convenience Methods (like NSRun***Alert)

/*!
 @brief    An improved version of Apple's -presentError: modal dialog.
 @details  Looks nicer than -presentError:, in my opinion.  Not all boldface.  Works in Panther.
 Presents the localizedFailureReason and localizedRecoverySuggestion, with a prefix.
 If an email address string has been given in the Info.plist key SSYAlert_SupportEmail, this dialog will
 set a "support" button (as described elsewhere) targetting this email.
 TODO: This should handle NSLocalizedRecoveryOptionsErrorKey and NSRecoveryAttempterErrorKey
 @param    error  The error to be presented.  If nil, this method does nothing
 @param    title  Title in boldface before small text, inside the window
 If nil, defaults to "Error:", localized.
 @param    button1Title  Title of button which, when clicked, causes method to return NSAlertDefaultReturn
 If nil, the title of button 1 is set to "OK", localized.
 @param    button2Title  Title of button which, when clicked, causes method to return NSAlertOtherReturn
 If nil, the second button does not appear.
 @param    button3Title  Title of button which, when clicked, causes method to return NSAlertAlternateReturn
 If nil, the third button does not appear.
 @result   NSAlertDefaultReturn, NSAlertAlternateReturn or NSAlertOtherReturn, depending on button clicked.
 */
+ (int)alertError:(NSError*)error
			title:(NSString*)title
		iconStyle:(int)iconStyle
	 button1Title:(NSString*)button1Title 
	 button2Title:(NSString*)button2Title 
	 button3Title:(NSString*)button3Title ;

/*!
 @brief    Invokes alertError:title:button1Title:button2Title:button3Title, passing nil for title
 */
+ (int)alertError:(NSError*)error
	 button1Title:(NSString*)button1Title 
	 button2Title:(NSString*)button2Title 
	 button3Title:(NSString*)button3Title ;

/*!
 @brief    Invokes alertError:button1Title:button2Title:button3Title, passing nil for the other arguments
 */
+ (int)alertError:(NSError*)error
	 button1Title:(NSString*)button1Title 
	 button2Title:(NSString*)button2Title ;

/*!
 @brief    Invokes alertError:button1Title:button2Title:button3Title, passing nil for the other arguments
 */
+ (int)alertError:(NSError*)error
	 button1Title:(NSString*)button1Title ;

/*!
 @brief    Invokes alertError:button1Title:button2Title:button3Title, passing nil for all arguments
 */
+ (void)alertError:(NSError*)error ;

/*!
 @brief    Runs a modal dialog with a message and 1-3 buttons

 @details  If title is nil, title will be set to localized "Alert"
 Pass 0-3 button title arguments.
 If 0, 1 button with localized "OK" will appear
 Must terminate buttons argument list with nil sentinel!
 */
+ (int)runModalDialogTitle:(NSString*)title
				   message:(NSString*)msg
				   buttons:(NSString*)button1Title, ... ;

#pragma mark * Methods to set appearance of controls

/*!
 @brief    Removes all subviews in the receiver and sets settings to default values
*/
- (void)cleanSlate ;

/*!
 @brief    Sets the title of the receiver's window

 @details  If not set, or set to nil, window title defaults to mainBundle's
 CFBundleName (which should be the localized name of the app)
*/
- (void)setWindowTitle:(NSString*)title ;

/*!
 @brief    Sets whether or not the receiver shows a progress bar
*/
- (void)setShowsProgressBar:(BOOL)showsProgressBar ;

/*!
 @brief    Sets the receiver's Title Text of the receiver to a given string.
 
 @details  Title Text appears in boldface near the top of the receiver
 window's content view.  It is not the window title.
*/
- (void)setTitleText:(NSString*)text ;

/*!
 @brief    Sets the receiver's Title Text to the localized word "Alert", stolen from NSGetAlertPanel

 @details  Title Text appears in boldface near the top of the receiver
 window's content view.  It is not the window title.
*/
- (void)setTitleToDefaultAlert ; 

/*!
 @brief    Sets the receiver's Small Text to the localized word "Alert", stolen from NSGetAlertPanel

 @details  Small Text is the text that appears in the window under the Title Text.
*/
- (void)setSmallText:(NSString*)text ;

/*!
 @brief    Sets the icon which will appear in the receiver
 @param    iconStyle  Pass one of the constants SSIconStyleNoIcon,
 SSIconStyleCritical, or SSIconStyleInformational
*/
- (void)setIconStyle:(int)iconStyle ;

/*!
 @brief    Sets the title of the (rightmost) "default" button, the
 one whose key equivalent is \r.
 @param    title  The desired button title, or nil.
*/
- (void)setButton1Title:(NSString*)title ;

/*!
 @brief    Sets the title of the (leftmost) "alternate" button, the
 one whose key equivalent is 'escape'.
 @details  Passing title as nil will remove the button.
 @param    title  The desired button title, or nil.
 */
- (void)setButton2Title:(NSString*)title ;  // key equivalent is escape

/*!
 @brief    Sets the title of the (middle) "other" button, the
 one with no key equivalent.
 @details  Passing title as nil will remove the button.
 @param    title  The desired button title, or nil.
 */
- (void)setButton3Title:(NSString*)title ;

/*!
 @brief    Sets the anchor in the application's Help Book which will
 be displayed in Help Viewer when the user clicks the question-mark
 "Help" button in the receiver's window.

 @details  Passing anchor as nil will remove the button.
 @param    anchor  The desired Help anchor
*/
- (void)setHelpAnchor:(NSString*)anchor ;

/*!
 @brief    Sets the email address behind a "Support" button, or removes the button.
 @details  Clicking the "Support" begins an email message with all of the error
 information pre-pasted into the body.  This is a square button which appears on the left
 side of the window.  Clicking it does not end the modal session.
 @param    The email address to which the message will be addressed.  If nil, removes the button.
 */
- (void)setSupportEmail:(NSString*)supportEmail ;

- (void)setCheckboxTitle:(NSString*)title ;

- (void)addOtherSubview:(NSView*)subview atIndex:(int)index ;
//- (void)addOtherSubview:(NSView <SSYHeightSizableToFit>*)subview atIndex:(int)index ;
- (void)removeOtherSubviewAtIndex:(int)index ;
- (void)removeAllOtherSubviews ;
- (int)otherSubviewsCount ;

// Other control details
- (void)setProgressBarShouldAnimate:(BOOL)yn ;
- (void)setIndeterminate:(BOOL)indeterminate ;
- (void)setMaxValue:(double)maxValue ;
- (void)setDoubleValue:(double)newValue ;
	// Will also display new value, if the previous display triggered
	// by invoking this method was more than .05 seconds ago.
- (void)incrementDoubleValueBy:(double)increment ;
- (void)incrementDoubleValueByObject:(NSNumber*)increment ;
- (void)setSmallTextAlignment:(NSTextAlignment)alignment ;
- (void)setMinTextFieldWidth:(float)minTextFieldWidth ;
	// Text Fields may be wider than set here if required
	// to accomodate buttons or checkbox title.
	// Default, initial value is 250.0.
- (void)setCheckboxState:(NSCellStateValue)state ;
- (void)setOKEnabled:(BOOL)enabled
			 toolTip:(NSString*)toolTip ;
	// Sets properties of button1, nominally the "OK" button.
// Direct access to subviews, for doing stuff not provided above
- (NSProgressIndicator*)progressBar ;
- (NSTextView*)titleTextView ;
- (NSTextView*)smallTextView ;

// Useful in creating otherSubviews
- (NSTextField*)smallTextFieldPrototype ;

// Saving and restoring view configurations
- (NSString*)pushConfiguration:(NSString*)identifier ;
	// If the alert window is currently visible, this method will push its
	// current configuration and return an identifier.  
	// If the alert window is not currently visible, this method will
	// do nothing and return nil.
	// If you pass an identifier argument, this method will append a sequence
	// number to it and return the concatenated identifier.
	// If you do not pass an identifier argument, or pass an empty string,
	// this method will return a string consisting only of the sequence number.
- (void)popConfiguration:(NSString*)identifier ;
	// If you pass nil as the identifier argument, or if the configurations
	// stack is empty, this method will end any modal session that might be
	// running and close the window (invokes -goAway).
	// If you pass an empty strying as the identifier argument, this method
	// will pop and display the top configuration if the stack is not empty.
	// If you pass a non-empty string as the identifier, this method will
	// check that the identifier of the popped configuration is equal to
	// the identifier argument.  If it is not, a warning will print to Console.
	// This method will also print a Console warning if you pass it a non-nil
	// identifier but there are no configurations to pop.
- (NSString*)peekTopIdentifier ;
	// For debugging, returns identifier of last configuration that
	// has been pushed onto the configurations stack.
- (void)doNotDisplayNextPop ;
	// If the popped configuration is not going to be shown, use this to avoid flicker.
- (NSDictionary*)configuration ;
	// Returns a dictionary of the current configuration.
	// This method is used for debugging.
	
// Displaying
- (void)setAllowsShrinking:(BOOL)allowsShrinking ;
	// Sets whether or not the titleTextView, smallTextView, and
	// otherSubviews are allowed to shrink when they are resized by
	// -resizeToFitAllowShrinking.  Does not control whether or not
	// the window as a whole shrinks or not.  So, if you want the
	// window layout to remain the same size, you must also not
	// add or remove subviews.  Reminder: -setTitleText:nil and
	// -setSmallText:nil will remove these respective subviews.
/*!
 @brief    Sets maximum number of characters, after which the title will be nicely truncated.
 @details  Truncation occurs by inserting an ellipsis 2/3 of the way through.  If this method is never invoked, defaults to 500.
 */
- (void)setTitleMaxChars:(int)value ;
/*!
 @brief    Sets maximum number of characters, after which the smallText will be nicely truncated.
 @details  Truncation occurs by inserting an ellipsis 2/3 of the way through.  If this method is never invoked, defaults to 1500.
 */
- (void)setSmallTextMaxChars:(int)value ;
- (void)display ;  // does a lay-out of subviews
// Must do this as the LAST thing
// after all controls and their details have been set.
- (void)show ;
- (BOOL)isVisible ;
- (void)kick ;
/*!
 @brief    Returns the receiver's window
 @details  This is a utility method for debugging.
*/
- (NSWindow*)window ;

// Running and Stopping
// Running as Modal Dialog
- (void)runModalDialog ;
	// only way to end modal dialog is for user
	// to click a button.  If the alert does not
	// currently have a button1, one will be
	// added with title [NSString localize:@"ok"]
// Running as Modal Session
- (void)runModalSession ;
	// If a modal session is already running, this does nothing.
- (void)endModalSession ;
	// If a modal session is not running, this does nothing.
- (void)setGoAwayWhenButtonClicked:(BOOL)yn ;
	// Initially, this defaults to NO
- (void)goAway ;
	// Ends any modal session that might be running and closes window

// Accessing results
- (BOOL)modalSessionRunning ;  
- (int)alertReturn ;
- (NSCellStateValue)checkboxState ;
//- (BOOL)isVisible ;

@end
